@@ -2,7 +2,7 @@
 
    make-iso.c: code to generate iso files
  
-   Copyright (C) 2002 Red Hat, Inc.
+   Copyright (C) 2002-2004 Red Hat, Inc.
   
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -24,12 +24,12 @@
 
 #include "config.h"
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 #include <sys/param.h>
 #include <sys/mount.h>
 #else
 #include <sys/vfs.h>
-#endif /* __FreeBSD__ */
+#endif /* __FreeBSD__ || __NetBSD__ || __OpenBSD__ */
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifdef HAVE_STATVFS
@@ -57,6 +57,7 @@ struct mkisofs_output {
 	int result;
 	int pid;
 	const char *filename;
+	gboolean debug;
 };
 
 static struct mkisofs_output *mkisofs_output_ptr;
@@ -444,11 +445,17 @@ mkisofs_stdout_read (GIOChannel   *source,
 		     GIOCondition  condition,
 		     gpointer      data)
 {
+	struct mkisofs_output *mkisofs_output = data;
 	char *line;
 	GIOStatus status;
 	
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
+
+	if (line && mkisofs_output->debug) {
+		g_print ("make_iso stdout: %s", line);
+	}
+
 	if (status == G_IO_STATUS_NORMAL) {
 		g_free (line);
 	}
@@ -465,21 +472,33 @@ mkisofs_stderr_read (GIOChannel   *source,
 	char fraction_str[7];
 	double fraction;
 	GIOStatus status;
-	
+
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
 
+	if (line && mkisofs_output->debug) {
+		g_print ("make_iso stderr: %s", line);
+	}
+
 	if (status == G_IO_STATUS_NORMAL) {
-		if (strncmp (line,"Total translation table size", 28) == 0) {
+		if (strncmp (line, "Total translation table size", 28) == 0) {
 			cd_progress_set_fraction (1.0);
 			g_main_loop_quit (mkisofs_output->loop);
 			mkisofs_output->result = RESULT_FINISHED;
 		}
-		
-		if (sscanf (line, "%6c%% done, estimate finish", fraction_str) == 1) {
-			fraction_str[6] = 0;
-			fraction = g_strtod (fraction_str, NULL);
-			cd_progress_set_fraction (fraction/100.0);
+
+		if (strstr (line, "estimate finish")) {
+			if (sscanf (line, "%6c%% done, estimate finish",
+						fraction_str) == 1) {
+				fraction_str[6] = 0;
+				fraction = g_strtod (fraction_str, NULL);
+				cd_progress_set_fraction (fraction/100.0);
+			}
+		}
+		if (strstr (line, "Incorrectly encoded string")) {
+			cd_progress_set_fraction (1.0);
+			g_main_loop_quit (mkisofs_output->loop);
+			mkisofs_output->result = RESULT_ERROR;
 		}
 
 		g_free (line);
@@ -492,7 +511,7 @@ mkisofs_stderr_read (GIOChannel   *source,
  */
 int
 make_iso (const char *filename, const char *label,
-		gboolean warn_low_space, gboolean use_joliet)
+		gboolean warn_low_space, gboolean use_joliet, gboolean debug)
 {
 	GnomeVFSURI *uri;
 	char *filelist = NULL;
@@ -521,6 +540,8 @@ make_iso (const char *filename, const char *label,
 		g_return_val_if_fail (strlen (label) < 32, RESULT_ERROR);
 	}
 
+	mkisofs_output.debug = debug;
+
 	dirname = g_strdup_printf ("iso-%s.XXXXXX", g_get_user_name ());
 	tempdir = g_build_filename (g_get_tmp_dir (), dirname, NULL);
 	g_free (dirname);
@@ -573,6 +594,9 @@ retry:
 	if (use_joliet) {
 		argv[i++] = "-J";
 	}
+	/* Undocumented -input-charset option */
+	argv[i++] = "-input-charset";
+	argv[i++] = "utf8";
 	argv[i++] = "-q";
 	argv[i++] = "-graft-points";
 	argv[i++] = "-path-list";
@@ -580,6 +604,16 @@ retry:
 	argv[i++] = "-print-size";
 	argv[i++] = NULL;
 
+	error = NULL;
+
+	if (debug) {
+		g_print ("launching command: ");
+		for (i = 0; argv[i] != NULL; i++) {
+			g_print ("%s ", argv[i]);
+		}
+		g_print ("\n");
+	}
+
 	if (!g_spawn_sync (NULL,
 			   (char **)argv,
 			   NULL,
@@ -588,7 +622,7 @@ retry:
 			   &stdout_data,
 			   &stderr_data,
 			   &exit_status,
-			   NULL)) {
+			   &error)) {
 		g_warning ("mkisofs command failed: %s\n", error->message);
 		g_error_free (error);
 		/* TODO: Better error handling */
@@ -652,6 +686,8 @@ retry:
 	if (use_joliet) {
 		argv[i++] = "-J";
 	}
+	argv[i++] = "-input-charset";
+	argv[i++] = "utf8";
 	argv[i++] = "-graft-points";
 	argv[i++] = "-path-list";
 	argv[i++] = filelist;
@@ -663,6 +699,14 @@ retry:
 	argv[i++] = filename;
 	argv[i++] = NULL;
 
+	if (debug) {
+		g_print ("launching command: ");
+		for (i = 0; argv[i] != NULL; i++) {
+			g_print ("%s ", argv[i]);
+		}
+		g_print ("\n");
+	}
+
 	cd_progress_set_text (_("Creating CD image"));
 	cd_progress_set_fraction (0.0);
 	cd_progress_set_image_spinning (TRUE);
